var scrollTopCheckerTimer = null;

function setup () {
    var body = document.getElementById("body");
    body.addEventListener("touchstart", touchStart, false);
    body.addEventListener("touchmove", touchMove, false);
    body.addEventListener("touchend", touchEnd, false);
    body.addEventListener("touchcancel", touchCancel, false);
    
    scrollTopCheckerTimer = setInterval("checkScrollActions()", 167);   //167 = 6x/sec
}

var old_height = 0;
var old_scroll_y = 0;
var caption = "";
var credit = "";

function updateImage(url, theCaption, theCredit) {
    var image = document.getElementById('article_image');
    if (image == null) {
        return;
    }
    old_height = image.offsetHeight;
    old_scroll_y = window.scrollY;
    
    caption = theCaption;
    credit = theCredit;
    
    // Swap images
    image.onload = correctImageScrollOffset;
    image.src = url;
    image.style.display = "inline";
}

function correctImageScrollOffset () {
    var image = document.getElementById('article_image');
    if (image == null) {
        return;
    }
    
    // Add caption and credit
    if (caption != "") {
        document.getElementById('caption').innerHTML = caption;
        caption = "";
    }
    
    if (credit != "") {
        document.getElementById('credit').innerHTML = credit;
        credit = "";
    }

    // Compensate for new scroll offset
    if (window.scrollY >= image.y) {
        var scroll_offset;
        if (old_height > image.offsetHeight) {
            scroll_offset = image.offsetHeight - old_height;
        }
        else {
            scroll_offset = old_height + image.offsetHeight;
        }
        window.scrollTo(0, old_scroll_y + scroll_offset);
    }
}


function setFontSize (size) {
    document.body.style.fontSize = size + "px";
}

function _getCurrentFontSize () {
    var body_elem = document.body;
    var font_size = body_elem.style.fontSize;

    // set fontSize to the computed style if this hasn't been done yet    
    if (font_size == null || font_size == "") {
        var default_font_size = document.defaultView.getComputedStyle(body_elem, null).getPropertyValue("font-size");
        document.body.style.fontSize = default_font_size;
        font_size = default_font_size;
    }
    
    return font_size.replace("px", "");
}

var lastTouchStart = 0;
var lastTouchMove = 0;
var lastTouchStartScrollPosition = 0;
var doubleTapTimer = -1;
var doubleTapThreshold = 500; // in milliseconds

function touchStart (event) {
    clearTimeout(doubleTapTimer);
    doubleTapTimer = -1;
    
    // Record scroll position at start of touch sequence
    lastTouchStartScrollPosition = window.scrollY;
    
    // For tap tracking, ignore double taps
    var new_start = (new Date()).getTime();
    if (new_start < lastTouchStart + doubleTapThreshold) {
        lastTouchStart = lastTouchMove = 0;
    }
    else {
        lastTouchStart = new_start;
    }
}

function touchMove (event) {
    lastTouchMove = (new Date()).getTime();
}

function touchEnd (event) {
    // Check for tap vs move/gesture.  Taps are touches that end without movement.
    clearTimeout(doubleTapTimer);
    doubleTapTimer = -1;
   
    // If it was a single tap, send tap callback
    if (lastTouchStart > lastTouchMove && lastTouchStartScrollPosition == window.scrollY) {
        doubleTapTimer = setTimeout("sendCommandToApp(\"touchEnd\")", doubleTapThreshold + 10);
    }
    lastTouchMove = 0;
}

function touchCancel (event) {
    lastTouchStart = 0;
    lastTouchMove = 0;
}

// Offset from top and bottom at which to always show the scroll bars
var scrollMargins = 75;

// Tracking changes between scroll checks
var lastScrollY = -1;
var lastScrollCommand = "";

// Enumeration to define valid values for scrollActionTaken
var scrollActionTop = 0;
var scrollActionBottom = 1;
var scrollActionMiddle = 2;

// This method watches for the "scroll to top" feature of the web view to be activated, since this
// cocoa trick does not trigger any JS events.  This executes rapidly -- needs to be  f a s t.
function checkScrollActions () {
    var currentScrollY = window.scrollY;
    var currentScrollCommand = "";
    
    // trigger scroll response once when we hit the _top_ of the page
    if (currentScrollY <= scrollMargins) {
        currentScrollCommand = "topScrollArea";
    }
    // trigger scroll response once when we hit the _bottom_ of the page
    else if (currentScrollY + window.innerHeight >= document.body.scrollHeight - scrollMargins) {
        currentScrollCommand = "bottomScrollArea";
    }
    // trigger response to general scrolling
    else if (currentScrollY != lastScrollY) {
        currentScrollCommand = "viewScrolled";
    }
    
    // Process action if scroll changed
    if (lastScrollY != -1 && currentScrollY != lastScrollY && lastScrollCommand != currentScrollCommand) {
        lastScrollCommand = currentScrollCommand;
        sendCommandToApp(currentScrollCommand);
    }
    lastScrollY = currentScrollY; // Record location for next run
}

// Sends a specially formatted command via the JS-to-ObjC bridge
function sendCommandToApp (command) {
    window.location = "NYTCommand://" + command;
}
